package edu.illinois.ncsa.paw.domain;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinTable;
import javax.persistence.OneToMany;

import org.springframework.data.mongodb.core.mapping.DBRef;

import edu.illinois.ncsa.domain.AbstractBean;
import edu.illinois.ncsa.domain.Dataset;
import edu.illinois.ncsa.domain.Person;

@Entity(name = "KNSGScenario")
public class KNSGScenario extends AbstractBean {

    /**
     * Serialization of object
     */
    private static final long serialVersionUID = 1L;
    private String            title            = "No Title";
    private String            description      = "No Description";
    /** Date the scenario is created */
    private Date              date             = new Date();
    private Person            creator          = null;

    @OneToMany(fetch = FetchType.LAZY, cascade = { CascadeType.MERGE })
    @JoinTable(name = "KNSGScenarioDatasets")
    @DBRef
    private List<Dataset>     datasets         = new ArrayList<Dataset>();

    // Can we wrap ogrescript workflows into these beans?
    // private List<PublishedWorkflowBean> publishedWorkflows = new
// ArrayList<PublishedWorkflowBean>();

    // The scenario had a reference to the RMI Service it was connected to, do
    // we
    // continue to make this part of the scenario?

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * Return the date when the scenario was created.
     * 
     * @return date the scenario was created.
     */
    public Date getDate() {
        return date;
    }

    /**
     * Sets the date when the scenario was created.
     * 
     * @param date
     *            sets the date when the scenario was created.
     * 
     */
    public void setDate(Date date) {
        this.date = date;
    }

    /**
     * Returns the PersonBean that is the creator of the scenario
     * 
     * @return PersonBean that represents the creator
     */
    public Person getCreator() {
        return creator;
    }

    /**
     * Sets the PersonBean that represents the creator of the scenario
     * 
     * @param creator
     *            the PersonBean that represents the creator of the scenario
     */
    public void setCreator(Person creator) {
        this.creator = creator;
    }

    public List<Dataset> getDatasets() {
        return datasets;
    }

    public void setDatasets(List<Dataset> datasets) {
        this.datasets.clear();
        if (datasets != null) {
            this.datasets.addAll(datasets);
        }
    }

    public void addDataset(Dataset dataset) {
        Iterator<Dataset> iterator = this.datasets.iterator();
        while (iterator.hasNext()) {
            Dataset tmp = iterator.next();
            if (tmp.getId().equals(dataset.getId())) {
                return;
            }
        }
        this.datasets.add(dataset);
    }

    public void removeDataset(Dataset dataset) {
        this.datasets.remove(dataset);
    }

}
